# -*- coding: utf-8 -*-

# flake8: noqa

import tinkerer
import tinkerer.paths

# **************************************************************
# TODO: Edit the lines below
# **************************************************************

# Change this to the name of your blog
project = 'My blog'

# Change this to the tagline of your blog
tagline = 'Add intelligent tagline here'

# Change this to the description of your blog
description = 'This is an awesome blog'

# Change this to your name
author = 'Winston Smith'

# Change this to your copyright string
copyright = '1984, ' + author

# Change this to your blog root URL (required for RSS feed)
website = 'http://127.0.0.1/blog/html/'

# **************************************************************
# More tweaks you can do
# **************************************************************

# Add your Disqus shortname to enable comments powered by Disqus
disqus_shortname = None

# Change your favicon (new favicon goes in _static directory)
html_favicon = '_static/tinkerer.ico'

# Pick another Tinkerer theme or use your own
html_theme = '{{ theme }}'

# Theme-specific options, see docs
html_theme_options = {}

# Link to RSS service like FeedBurner if any, otherwise feed is
# linked directly
rss_service = None

# Generate full posts for RSS feed even when using "read more"
rss_generate_full_posts = False

# Number of blog posts per page
posts_per_page = 10

# Character use to replace non-alphanumeric characters in slug
slug_word_separator = '_'

# Set to page under /pages (eg. "about" for "pages/about.html")
landing_page = None

# Set to override the default name of the first page ("Home")
first_page_title = None

# **************************************************************
# Edit lines below to further customize Sphinx build
# **************************************************************

# Add other Sphinx extensions here
extensions = [{{ extensions }}]

# Add other template paths here
templates_path = ['_templates']

# Add other static paths here
html_static_path = ['_static', tinkerer.paths.static]

# Add other theme paths here
html_theme_path = ['_themes', tinkerer.paths.themes]

# Add file patterns to exclude from build
exclude_patterns = ['drafts/*', '_templates/*']

# Add templates to be rendered in sidebar here
html_sidebars = {
    '**': ['recent.html', 'searchbox.html']
}

# Add an index to the HTML documents.
html_use_index = False

# **************************************************************
# Do not modify below lines as the values are required by
# Tinkerer to play nice with Sphinx
# **************************************************************

source_suffix = tinkerer.source_suffix
master_doc = tinkerer.master_doc
version = tinkerer.__version__
release = tinkerer.__version__
html_title = project
html_show_sourcelink = False
html_add_permalinks = ''



'''
    aggregator
    ~~~~~~~~~~

    Aggregates multiple posts into single pages.

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENSE file
'''
import copy
from tinkerer.ext.uistr import UIStr


def make_aggregated_pages(app):
    '''
    Generates aggregated pages.
    '''
    env = app.builder.env
    posts_per_page = app.config.posts_per_page
    landing_page = app.config.landing_page

    if landing_page:
        if "pages/%s" % landing_page not in env.blog_pages:
            raise Exception("Missing landing page: %s" % landing_page)

        yield (
            "index",
            {"redirect_url": "./pages/%s.html" % landing_page},
            "index.html"
        )

    # get post groups
    groups = [env.blog_posts[i:i+posts_per_page]
              for i in range(0, len(env.blog_posts), posts_per_page)]

    # for each group
    for i, posts in enumerate(groups):
        # initialize context
        context = {
            "prev": {},
            "next": {},
            "posts": []
        }

        # add posts to context
        for post in posts:
            # deepcopy metadata for each post
            metadata = copy.deepcopy(env.blog_metadata[post])
            context["posts"].append(metadata)

        pagename = "page%d" % (i + 1)

        # handle navigation
        if i == 0:
            # if landing_page, keep "page1", else use "index" so we land on
            # first aggregated page
            if not landing_page:
                pagename = "index"

            # first page doesn't have prev link and its title is "Home"
            context["prev"] = None
            context["title"] = UIStr.HOME
        else:
            # following pages prev-link to previous page (titled as "Newer")
            context["prev"]["title"] = UIStr.NEWER
            context["prev"]["link"] = "page%d.html" % i

            # if no landing_page, prev link for second page is "index"
            if i == 1 and not landing_page:
                context["prev"]["link"] = "index.html"

            context["title"] = UIStr.PAGE_FMT % (i + 1)

        if i == len(groups) - 1:
            # last page doesn't have next link
            context["next"] = None
        else:
            # other pages next-link to following page (titled as "Older")
            context["next"]["title"] = UIStr.OLDER
            context["next"]["link"] = "page%d.html" % (i + 2)

        context["archive_title"] = UIStr.BLOG_ARCHIVE

        yield (pagename, context, "aggregated.html")



'''
    draft
    ~~~~~

    Handles creating drafts.

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENCE file
'''
import os
import re
import shutil
import tinkerer
from tinkerer import master, paths, utils, writer


def create(title, template=None):
    '''
    Creates a new post draft.
    '''
    name = utils.name_from_title(title)
    template = template or paths.post_template

    path = os.path.join(
        utils.get_path(paths.root, "drafts"),
        name + tinkerer.source_suffix,
    )

    if os.path.exists(path):
        raise Exception("Draft '%s' already exists at '%s" %
                        (title, path))

    writer.render(template, path,
                  {"title":      title,
                   "content":    "",
                   "author":     "default",
                   "categories": "none",
                   "tags":       "none"})

    return path


def move(path):
    '''
    Demotes given file to draft.
    '''
    # get dirname and filename
    dirname, filename = os.path.split(path)

    # get docname without extension
    docname = os.path.splitext(filename)[0]

    draft = os.path.join(utils.get_path(paths.root, "drafts"), filename)

    # move file
    shutil.move(path, draft)

    # check if file is a post or a page
    if os.path.basename(dirname) == "pages":
        docname = "pages/" + docname
    else:
        match = re.match(r".*(?P<y>\d{4}).(?P<m>\d{2}).(?P<d>\d{2})$", dirname)
        if not match:
            return draft
        g = match.group
        docname = "/".join([g("y"), g("m"), g("d"), docname])

    # remove file from TOC
    master.remove_doc(docname)

    return draft



'''
    rss
    ~~~

    RSS feed generator for blog.

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENSE file
'''
import email.utils
import time

import pyquery

from tinkerer.ext import patch


def remove_header_link(body):
    """Remove any headerlink class anchor tags from the body.
    """
    doc = pyquery.PyQuery(body)
    doc.remove('a.headerlink')
    body = doc.html()
    return body


def add_rss(app, context):
    '''
    Adds RSS service link to page context.
    '''
    context["rss_service"] = app.config.rss_service


def generate_feed(app):
    '''
    Generates RSS feed.
    '''
    env = app.builder.env

    # don't do anything if no posts are available
    if not env.blog_posts:
        return

    posts = env.blog_posts
    if app.config.rss_max_items > 0:
        posts = posts[:app.config.rss_max_items]

    context = make_feed_context(app, None, posts)
    yield ("rss", context, "rss.html")


def make_feed_context(app, feed_name, posts):
    env = app.builder.env
    context = dict()

    # feed items
    context["items"] = []
    for post in posts:
        link = "%s%s.html" % (app.config.website, post)

        timestamp = email.utils.formatdate(
            time.mktime(env.blog_metadata[post].date.timetuple()),
            localtime=True)

        categories = [category[1] for category in
                      env.blog_metadata[post].filing["categories"]]

        description = patch.strip_xml_declaration(
            patch.patch_links(
                env.blog_metadata[post].body,
                # first 11 characters of post is the path (YYYY/MM/DD/)
                app.config.website + post[:11],
                # following characters represent filename
                post[11:],
                replace_read_more_link=not app.config.rss_generate_full_posts,
            ),
        )
        description = remove_header_link(description)

        context["items"].append({
            "title": env.titles[post].astext(),
            "link": link,
            "description": description,
            "categories": categories,
            "pubDate": timestamp
        })

    # feed metadata
    if feed_name:
        context["title"] = "%s - %s" % (app.config.project, feed_name)
    else:
        context["title"] = app.config.project
    context["link"] = app.config.website
    context["tagline"] = app.config.tagline
    context["language"] = "en-us"

    # feed pubDate is equal to latest post pubDate
    if context['items']:
        context["pubDate"] = context["items"][0]["pubDate"]

    return context



'''
    patch
    ~~~~~

    Handles HTML link patching for images and cross-references. Sphinx
    generates these links as relative paths - aggregated pages and RSS
    feed require these to be patched.

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENSE file
'''
from os import path

import pyquery

from tinkerer.ext.uistr import UIStr


def patch_aggregated_metadata(context):
    """
    Patches context in aggregated pages
    """
    for metadata in context["posts"]:
        metadata.body = patch_links(
            metadata.body,
            metadata.link[:11],  # first 11 characters is path (YYYY/MM/DD/)
            metadata.link[11:],  # following characters represent filename
            True)      # hyperlink title to post
        metadata.body = strip_xml_declaration(metadata.body)


def patch_links(body, docpath, docname=None, link_title=False,
                replace_read_more_link=True):
    '''
    Parses the document body and calls patch_node from the document root
    to fix hyperlinks. Also hyperlinks document title. Returns resulting
    XML as string.
    '''
    doc = pyquery.PyQuery(body)
    patch_node(doc, docpath, docname)

    body = doc.html()
    if docname and replace_read_more_link:
        body = make_read_more_link(body, docpath, docname)

    if link_title:
        return hyperlink_title(body, docpath, docname)
    else:
        return body


def hyperlink_title(body, docpath, docname):
    """
    Hyperlink titles by embedding appropriate a tag inside
    h1 tags (which should only be post titles).
    """
    body = body.replace("<h1>", '<h1><a href="%s.html">' %
                        (docpath + docname), 1)
    body = body.replace("</h1>", "</a></h1>", 1)
    return body


def make_read_more_link(body, docpath, docname):
    """
    Create "read more" link if marker exists.
    """
    doc = pyquery.PyQuery(body)
    link_p = ('<p class="readmorewrapper"><a class="readmore" '
              'href="%s.html#more">%s</a></p>' %
              (docpath + docname, UIStr.READ_MORE))
    doc('div#more').replaceWith(link_p)
    doc('p.readmorewrapper').next_all().remove()
    doc('p.readmorewrapper').parents().nextAll().remove()
    return doc.html()


def collapse_path(path_url):
    '''
    Normalize relative path and patch protocol prefix
    and Windows path separator
    '''
    return path.normpath(path_url).replace("\\", "/").replace(":/", "://")


def patch_node(node, docpath, docname=None):
    for img in node.find('img'):
        src = img.get('src', '')
        if src.startswith(".."):
            src = docpath + src
        src = collapse_path(src)
        img.set('src', src)

    for anchor in node.find('a'):
        ref = anchor.get('href')
        # skip anchor links <a name="anchor1"></a>, <a name="more"/>
        if ref is not None:
            # patch links only - either starting with "../" or having
            # "internal" class
            is_relative = ref.startswith("../")

            classes = anchor.get('class')
            is_internal = classes and "internal" in classes

            if not is_relative and not is_internal:
                continue

            ref = docpath + ref

            # html anchor with missing post.html
            # e.g. href="2012/08/23/#the-cross-compiler"
            # now href="2012/08/23/a_post.html#the-cross-compiler"
            ref = ref.replace("/#", "/%s.html#" % docname)

            # normalize urls so "2012/08/23/../../../_static/" becomes
            # "_static/" - we can use normpath for this, just make sure
            # to revert change on protocol prefix as normpath deduplicates
            # // (http:// becomes http:/)
            ref = collapse_path(ref)
            anchor.set('href', ref)


def strip_xml_declaration(body):
    """
    Remove XML declaration from document body.
    """
    return body.replace('<?xml version="1.0" ?>', '')



'''
    writer
    ~~~~~~

    Internal template writer - handles template rendering and blog setup.

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENSE file
'''

from jinja2 import ChoiceLoader, Environment, FileSystemLoader, PackageLoader
import os
import shutil
from tinkerer import paths, utils


# jinja environment
env = Environment(loader=ChoiceLoader([
    # first choice is _templates subdir from blog root
    FileSystemLoader(paths.templates),
    # if template is not there, use tinkerer builtin
    PackageLoader("tinkerer", "__templates")]))


def render(template, destination, context={}, safe=False):
    '''
    Renders the given template at the given destination with the given context.
    '''
    with open(destination, "wb") as dest:
        dest.write(env.get_template(template).render(context).encode("utf8"))


def render_safe(template, destination, context={}):
    '''
    Similar to render but only renders the template if the destination doesn't
    already exist.
    '''
    # if safe is set to True, abort if file already exists
    if os.path.exists(destination):
        return False

    render(template, destination, context)

    return True


def write_master_file():
    '''
    Writes the blog master document.
    '''
    return render_safe("master.rst", paths.master_file)


def write_index_file():
    '''
    Writes the root index.html file.
    '''
    return render_safe(
        "index.html",
        paths.index_file,
        {"redirect_url": "./blog/html/index.html"}
    )


'''
Default Tinkerer extensions.
'''
DEFAULT_EXTENSIONS = [
    "tinkerer.ext.blog",
    "tinkerer.ext.disqus"
]


def write_conf_file(extensions=DEFAULT_EXTENSIONS, theme="flat"):
    '''
    Writes the Sphinx configuration file.
    '''
    return render_safe(
        "conf.py", paths.conf_file,
        {"extensions": ", ".join(["'%s'" % ext for ext in extensions]),
         "theme": theme})


def copy_templates():
    '''
    Copies Tinkerer post and page templates to blog _templates directory.
    '''
    for template in [paths.post_template, paths.page_template]:
        if not os.path.exists(os.path.join(paths.root, "_templates",
                                           template)):
            shutil.copy(
                os.path.join(paths.__internal_templates_abs_path, template),
                os.path.join(paths.root, "_templates")
            )


def copy_static():
    '''
    Copies Tinkerer favicon to blog _static directory.
    '''
    if not os.path.exists(os.path.join(paths.root, "_static", paths.favicon)):
        shutil.copy(
            os.path.join(paths.static, paths.favicon),
            os.path.join(paths.root, "_static")
        )


def setup_blog():
    '''
    Sets up a new blog.
    '''
    utils.get_path(paths.root, "_static")
    utils.get_path(paths.root, "_templates")
    utils.get_path(paths.root, "drafts")
    copy_templates()
    copy_static()
    write_master_file()
    write_index_file()
    return write_conf_file()



'''
    Tinkerer command line
    ~~~~~~~~~~~~~~~~~~~~~

    Automates the following blog operations:

    setup - to create a new blog
    build - to clean build blog
    post - to create a new post
    page - to create a new page

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENSE file
'''
import argparse
from datetime import datetime
import os
import shutil
import subprocess
import tinkerer
from tinkerer import draft, output, page, paths, post, writer


def setup():
    '''
    Sets up a new blog in the current directory.
    '''
    # it is a new blog if conf.py doesn't already exist
    new_blog = writer.setup_blog()

    output.filename.info("conf.py")
    if new_blog:
        output.write.info("Your new blog is almost ready!")
        output.write.info("You just need to edit a couple of lines in %s" %
                          (os.path.relpath(paths.conf_file), ))
    else:
        output.write.info("Done")


def build():
    '''
    Runs a clean Sphinx build of the blog.
    '''
    # clean build directory
    if os.path.exists(paths.blog):
        shutil.rmtree(paths.blog)

    flags = ["sphinx-build"]
    # silence Sphinx if in quiet mode
    if output.quiet:
        flags.append("-q")
    flags += ["-d", paths.doctree, "-b", "html", paths.root, paths.html]

    # build always prints "index.html"
    output.filename.info("index.html")

    # copy some extra files to the output directory
    if os.path.exists("_copy"):
        shutil.copytree("_copy/", paths.html)

    return subprocess.call(flags)


def create_post(title, date, template):
    '''
    Creates a new post with the given title or makes an existing file a post.
    '''
    move = os.path.exists(title)

    if move:
        new_post = post.move(title, date)
    else:
        new_post = post.create(title, date, template)

    output.filename.info(new_post.path)
    if move:
        output.write.info("Draft moved to post '%s'" % new_post.path)
    else:
        output.write.info("New post created as '%s'" % new_post.path)


def create_page(title, template):
    '''
    Creates a new page with the given title or makes an existing file a page.
    '''
    move = os.path.exists(title)

    if move:
        new_page = page.move(title)
    else:
        new_page = page.create(title, template)

    output.filename.info(new_page.path)
    if move:
        output.write.info("Draft moved to page '%s'" % new_page.path)
    else:
        output.write.info("New page created as '%s'" % new_page.path)


def create_draft(title, template):
    '''
    Creates a new draft with the given title or makes an existing file a draft.
    '''
    move = os.path.exists(title)

    if move:
        new_draft = draft.move(title)
    else:
        new_draft = draft.create(title, template)

    output.filename.info(new_draft)
    if move:
        output.write.info("File moved to draft '%s'" % new_draft)
    else:
        output.write.info("New draft created as '%s'" % new_draft)


def preview_draft(draft_file):
    '''
    Rebuilds the blog, including the given draft.
    '''
    if not os.path.exists(draft_file):
        raise Exception("Draft named '%s' does not exist" % draft_file)

    # promote draft
    preview_post = post.move(draft_file)

    try:
        # rebuild
        result = build()
    finally:
        # demote post back to draft
        draft.move(preview_post.path)

    return result


def main(argv=None):
    '''
    Parses command line and executes required action.
    '''
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group()
    group.add_argument("-s", "--setup", action="store_true",
                       help="setup a new blog")
    group.add_argument("-b", "--build", action="store_true", help="build blog")
    group.add_argument(
        "-p", "--post", nargs=1,
        help="create a new post with the title POST (if a file named POST "
        "exists, it is moved to a new post instead)")
    group.add_argument(
        "--page", nargs=1,
        help="create a new page with the title PAGE (if a file named PAGE "
        "exists, it is moved to a new page instead)")
    group.add_argument(
        "-d", "--draft", nargs=1,
        help="creates a new draft with the title DRAFT (if a file named DRAFT "
        "exists, it is moved to a new draft instead)")
    group.add_argument(
        "--preview", nargs=1,
        help="rebuilds the blog, including the draft PREVIEW, without "
        "permanently promoting the draft to a post")
    group.add_argument(
        "-v", "--version", action="store_true",
        help="display version information")

    parser.add_argument(
        '-t', '--template', action='store', default=None,
        help="specify a body template, defaults to page or post",
    )
    parser.add_argument(
        "--date", nargs=1,
        help="optionally specify a date as 'YYYY/mm/dd' for the post, "
        "useful when migrating blogs; can only be used together with "
        "-p/--post")

    group = parser.add_mutually_exclusive_group()
    group.add_argument("-q", "--quiet", action="store_true", help="quiet mode")
    group.add_argument(
        "-f", "--filename", action="store_true",
        help="output filename only - useful to pipe Tinkerer commands")

    command = parser.parse_args(argv)

    output.init(command.quiet, command.filename)

    # tinkerer should be run from the blog root unless in setup mode or -v
    if (not command.setup and not command.version and not
            os.path.exists(paths.conf_file)):
        output.write.error("Tinkerer must be run from your blog root "
                           "(directory containing 'conf.py')")
        return -1

    post_date = None
    if command.date:
        # --date only works with --post
        if not command.post:
            output.write.error("Can only use --date with -p/--post.")
            return -1

        try:
            post_date = datetime.strptime(command.date[0], "%Y/%m/%d")
        except Exception:
            output.write.error(
                "Invalid post date: format should be YYYY/mm/dd"
            )
            return -1

    if command.template:
        if not os.path.exists(os.path.join(paths.templates, command.template)):
            output.write.error(
                "The specified template does not exist. "
                " Make sure the template is placed inside the _templates"
                " subdirectory of your blog.")
            return -1

    if command.setup:
        setup()
    elif command.build:
        return build()
    elif command.post:
        create_post(command.post[0], post_date, command.template)
    elif command.page:
        create_page(command.page[0], command.template)
    elif command.draft:
        create_draft(command.draft[0], command.template)
    elif command.preview:
        preview_draft(command.preview[0])
    elif command.version:
        output.write.info("Tinkerer version %s" % tinkerer.__version__)
    else:
        parser.print_help()

    return 0



"""
    html5
    ~~~~~

    Monkey-patch Sphinx HTML translator to emit HTML5.

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENSE file.
"""
from sphinx.writers.html import HTMLTranslator


def visit_desc_addname(self, node):
    '''
    Similar to Sphinx but using a <span> node instead of <tt>.
    '''
    self.body.append(self.starttag(node, 'span', '', CLASS='descclassname'))


def depart_desc_addname(self, node):
    '''
    Similar to Sphinx but using a <span> node instead of <tt>.
    '''
    self.body.append('</span>')


def visit_desc_name(self, node):
    '''
    Similar to Sphinx but using a <span> node instead of <tt>.
    '''
    self.body.append(self.starttag(node, 'span', '', CLASS='descname'))


def depart_desc_name(self, node):
    '''
    Similar to Sphinx but using a <span> node instead of <tt>.
    '''
    self.body.append('</span>')


def visit_literal(self, node):
    '''
    Similar to Sphinx but using a <span> node instead of <tt>.
    '''
    self.body.append(self.starttag(node, 'span', '',
                                   CLASS='docutils literal'))
    self.protect_literal_text += 1


def depart_literal(self, node):
    '''
    Similar to Sphinx but using a <span> node instead of <tt>.
    '''
    self.protect_literal_text -= 1
    self.body.append('</span>')


def patch_translator():
    '''
    Monkey-patch Sphinx translator to emit proper HTML5.
    '''
    HTMLTranslator.visit_desc_addname = visit_desc_addname
    HTMLTranslator.depart_desc_addname = depart_desc_addname
    HTMLTranslator.visit_desc_name = visit_desc_name
    HTMLTranslator.depart_desc_name = depart_desc_name
    HTMLTranslator.visit_literal = visit_literal
    HTMLTranslator.depart_literal = depart_literal



'''
    Tinkerer output
    ~~~~~~~~~~~~~~~

    Handles writing output

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENSE file
'''
import logging
import sys


# output writer
write = logging.getLogger("write")


# used in "filename only" mode
filename = logging.getLogger("filename")


# global quiet is used to pass -q to Sphinx build
quiet = False


def init(quiet_mode, filename_only):
    """
    Initialize output based on quiet/filename-only flags
    """
    global quiet

    # global quiet is used to pass -q to Sphinx build so it should be set when
    # either in quiet mode or filename-only mode
    quiet = quiet_mode or filename_only

    # always handle write as it also output all errors
    write.addHandler(logging.StreamHandler())

    if filename_only:
        # in filename-only mode, also handle filename and suppress other
        # messages below ERROR level
        filename.addHandler(logging.StreamHandler(sys.stdout))
        write.setLevel(logging.ERROR)
        filename.setLevel(logging.INFO)
    elif quiet:
        # in quiet mode, only display ERROR and above
        write.setLevel(logging.ERROR)
    else:
        # otherwise display INFO
        write.setLevel(logging.INFO)



'''
    post
    ~~~~

    Handles creating new posts and inserting them in the master document.

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENCE file
'''
import os
import shutil
import tinkerer
from tinkerer import master, paths, utils, writer


class Post():
    '''
    The class provides methods to create a new post and insert it into the
    master document.
    '''

    def __init__(self, title=None, path=None, date=None):
        '''
        Initializes a new post and creates path to it if it doesn't already
        exist.
        '''
        self.title = title

        # get year, month and day from date
        self.year, self.month, self.day = utils.split_date(date)

        # get name from path if specified, otherwise from title
        if path:
            self.name = utils.name_from_path(path)
        else:
            self.name = utils.name_from_title(title)

        # create post directory if it doesn't exist and get post path
        self.path = os.path.join(
            utils.get_path(
                paths.root,
                self.year,
                self.month,
                self.day),
            self.name) + tinkerer.source_suffix

        # docname as it should appear in TOC
        self.docname = "/".join([self.year, self.month, self.day, self.name])

    def write(self, content="", author="default",
              categories="none", tags="none",
              template=None):
        '''
        Writes the post template with given arguments.
        '''
        template = template or paths.post_template
        writer.render(template, self.path,
                      {"title":      self.title,
                       "content":    content,
                       "author":     author,
                       "categories": categories,
                       "tags":       tags})


def create(title, date=None, template=None):
    '''
    Creates a new post given its title.
    '''
    post = Post(title, path=None, date=date)
    if os.path.exists(post.path):
        raise Exception("Post '%s' already exists at '%s" %
                        (title, post.path))

    post.write(template=template)
    if not master.exists_doc(post.docname):
        master.prepend_doc(post.docname)
    return post


def move(path, date=None):
    '''
    Moves a post given its path.
    '''
    post = Post(title=None, path=path, date=date)
    if os.path.exists(post.path):
        raise Exception("Post '%s' already exists" %
                        (post.path,))
    shutil.move(path, post.path)
    if not master.exists_doc(post.docname):
        master.prepend_doc(post.docname)
    return post



'''
    utils
    ~~~~~

    Tinkerer utility functions.

    :copyright: Copyright 2011-2018 by Vlad Riscutia and contributors (see
    CONTRIBUTORS file)
    :license: FreeBSD, see LICENSE file
'''
import datetime
import imp
import os
import re


UNICODE_ALNUM_PTN = re.compile(r"[\W_]+", re.U)


def name_from_title(title):
    '''
    Returns a doc name from a title by replacing all groups of
    characters which are not alphanumeric or '_' with the word
    separator character.
    '''
    try:
        word_sep = get_conf().slug_word_separator
    except Exception:
        word_sep = "_"

    return UNICODE_ALNUM_PTN.sub(word_sep, title).lower().strip(word_sep)


def name_from_path(path):
    '''
    Returns a doc name from a path by extracting the filename without
    extension.
    '''
    return os.path.splitext(os.path.basename(path))[0]


def get_path(*args):
    '''
    Creates a path if it doesn't already exist.
    '''
    path = os.path.join(*args)
    if not os.path.exists(path):
        os.makedirs(path)
    return path


def split_date(date=None):
    '''
    Splits a date into formatted year, month and day strings. If not date is
    provided, current date is used.
    '''
    if not date:
        date = datetime.datetime.today()

    return "%04d" % date.year, "%02d" % date.month, "%02d" % date.day


def get_conf():
    '''
    Import conf.py from current directory.
    '''
    return imp.load_source("conf", "./conf.py")
